@@ -4,13 +4,14 @@
 #include <boost/tuple/tuple_comparison.hpp>
 #include <boost/multi_index/key_extractors.hpp>
 #include <boost/multi_index/sequenced_index.hpp>
+#include <boost/foreach.hpp>
 #include <iostream>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string>
 #include <sstream>
 
-using namespace std;
+#include "namespaces.hh"
 int main()
 {
         ostringstream str;
